#pragma once
/*
 *      Copyright (C) 2005-2008 Team XBMC
 *      http://www.xbmc.org
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XBMC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#include "IDirectory.h"
#include "DynamicDll.h"
#include <hdhomerun/hdhomerun.h>
#include "FileSystem/IFile.h"

class DllHdHomeRunInterface
{
public:
  virtual ~DllHdHomeRunInterface() {}
  virtual int           discover_find_devices_custom(uint32_t target_ip, uint32_t device_type, uint32_t device_id, struct hdhomerun_discover_device_t result_list[], int max_count)=0;
  virtual struct hdhomerun_device_t*  device_create_from_str(const char *device_str, struct hdhomerun_debug_t *dbg)=0;
  virtual void          device_destroy(struct hdhomerun_device_t *hd)=0;
  virtual int           device_stream_start(struct hdhomerun_device_t *hd)=0;
  virtual uint8_t*      device_stream_recv(struct hdhomerun_device_t *hd, size_t max_size, size_t* pactual_size)=0;
  virtual void          device_stream_stop(struct hdhomerun_device_t *hd)=0;
  virtual int           device_set_tuner_channel(struct hdhomerun_device_t *hd, const char *channel)=0;
  virtual int           device_set_tuner_program(struct hdhomerun_device_t *hd, const char *program)=0;
  virtual int           device_set_tuner_from_str(struct hdhomerun_device_t *hd, const char *tuner_str)=0;
  virtual void          device_set_tuner(struct hdhomerun_device_t *hd, unsigned int tuner)=0;
  virtual int           device_get_tuner_status(struct hdhomerun_device_t *hd, char **pstatus_str, struct hdhomerun_tuner_status_t *status)=0;
};

class DllHdHomeRun : public DllDynamic, public DllHdHomeRunInterface
{
public:
  virtual ~DllHdHomeRun () {};

  virtual int discover_find_devices_custom (uint32_t p1, uint32_t p2, uint32_t p3, struct hdhomerun_discover_device_t p4[], int p5)
      { return ::hdhomerun_discover_find_devices_custom (p1, p2, p3, p4, p5); }

  virtual struct hdhomerun_device_t *device_create_from_str (const char* p1, struct hdhomerun_debug_t *p2)
      { return ::hdhomerun_device_create_from_str (p1, p2); }

  virtual void device_destroy (struct hdhomerun_device_t* p1)
      { ::hdhomerun_device_destroy (p1); }

  virtual int device_stream_start (struct hdhomerun_device_t* p1)
      { return ::hdhomerun_device_stream_start (p1); }


  virtual uint8_t *device_stream_recv (struct hdhomerun_device_t* p1, size_t p2, size_t* p3)
      { 
        uint8_t *retval;

        size_t tmp = *p3;
        retval = ::hdhomerun_device_stream_recv (p1, p2, &tmp);
        *p3 = tmp;

        return retval;
      }

  virtual void device_stream_stop (struct hdhomerun_device_t* p1)
      { ::hdhomerun_device_stream_stop (p1); }

  virtual int device_set_tuner_channel (struct hdhomerun_device_t *p1, const char *p2)
      { return ::hdhomerun_device_set_tuner_channel (p1, p2); }

  virtual int device_set_tuner_program (struct hdhomerun_device_t *p1, const char *p2)
      { return ::hdhomerun_device_set_tuner_program (p1, p2); }


  virtual int device_set_tuner_from_str (struct hdhomerun_device_t *p1, const char *p2)
      { return ::hdhomerun_device_set_tuner_from_str (p1, p2); }


  virtual void device_set_tuner (struct hdhomerun_device_t *p1, unsigned int p2)
      { ::hdhomerun_device_set_tuner (p1, p2); }

  virtual int device_get_tuner_status (struct hdhomerun_device_t *p1, char **p2, struct hdhomerun_tuner_status_t *p3)
      { return ::hdhomerun_device_get_tuner_status (p1, p2, p3); }

  // DLL faking.
  virtual bool ResolveExports() { return true; }
  virtual bool Load() { return true; }
  virtual void Unload() {}
};

namespace XFILE
{
  class CDirectoryHomeRun : public IDirectory
  {
    public:
      CDirectoryHomeRun(void);
      virtual ~CDirectoryHomeRun(void);
      virtual bool IsAllowed(const CStdString &strFile) const { return true; };
      virtual bool GetDirectory(const CStdString& strPath, CFileItemList &items);
    private:
      DllHdHomeRun m_dll;
  };
}

namespace XFILE
{
  class CFileHomeRun : public IFile
  {
    public:
      CFileHomeRun();
      ~CFileHomeRun();

      virtual bool          Exists(const CURL& url);
      virtual int64_t       Seek(int64_t iFilePosition, int iWhence);
      virtual int           Stat(const CURL& url, struct __stat64* buffer);
      virtual int64_t       GetPosition();
      virtual int64_t       GetLength();

      virtual bool          Open(const CURL& url);
      virtual void          Close();
      virtual unsigned int  Read(void* lpBuf, int64_t uiBufSize);
      virtual int           GetChunkSize();
    private:
      struct hdhomerun_device_t* m_device;
      DllHdHomeRun m_dll;
  };
}
